class SimController {
  model;
  timekeeper;

  simManager;
  taskManager;
  splash;

  constructor() {}

  loadSim(title) {
    return this.simManager.loadSim(title);
  }

  loadTask(title) {
    return this.taskManager.loadTask(title);
  }

  loadSplash() {
    if (this.simManager && this.simManager.sims) {
      this.simManager.sims.forEach((sim) => sim.close());
    }
    this.splash = parseSplash(this.model.splash, this.timekeeper.running);
    $("#main").html(this.splash);
  }

  loadFirstSim() {
    // BUG: strange time delay, product is highlighted half a second before data type
    $(`#data-menu .show`).removeClass("show");
    $(`#data_type_label-0`).prop("checked", true);
    $(`#data_type-0`).addClass("show");
    $(`#product-${this.model.data_types[0].products[0].name}`).prop(
      "checked",
      true
    );

    return this.loadSim(this.model.data_types[0].products[0].name);
  }

  init() {
    this.model = window.model;

    /* set timekeeper instance */
    this.timekeeper = new Timekeeper(
      this.model.time.bounds.start,
      this.model.time.bounds.end,
      this.model.time.speed,
      this.model.time.revealDates
    );

    this.loadSplash();

    this.simManager = new SimManager(this.model);

    if (this.model.metadata.showHeading) {
      $("#simulator-header-title").html(this.model.metadata.name);
      $("#header").attr("style", "display: block; width: 100%;");
    }

    let dataTypeList = parseDataTypes(this.model.data_types);
    $("#topDataMenu").html(dataTypeList);

    /* set tasks */
    this.taskManager = new TaskManager(this.model.tasks);
  }

  start() {
    this.timekeeper.start();
    this.simManager.subscribe(
      this.timekeeper.clockTickObservable,
      this.timekeeper.newTimeObservable
    );
    this.taskManager.subscribe(
      this.timekeeper.clockTickObservable,
      this.timekeeper.newTimeObservable
    );
  }

  pause() {
    this.timekeeper.pause();
    this.simManager.unsubscribe();
    this.taskManager.unsubscribe();
  }

  getCurrentTime() {
    return this.timekeeper.currentTime;
  }

  getStartTime() {
    return this.timekeeper.startTime;
  }

  getEndTime() {
    return this.timekeeper.endTime;
  }

  setSimulationTime(date) {
    return this.timekeeper.setSimulationTime(date);
  }
}
